/********************************************************/
/*  Profile  Program                                    */
/*   This program demonstrates some of the classes      */
/*   for maintaining OS/2 profiles                      */
/********************************************************/

#include <iapp.hpp>
#include <istattxt.hpp>
#include <imenubar.hpp>
#include <ipopmenu.hpp>
#include <ifont.hpp>
#include <ifiledlg.hpp>
#include <istring.hpp>
#include <ibuffer.hpp>
#include <ilistbox.hpp>
#include <isetcv.hpp>
#include <isplitcv.hpp>
#include <ipushbut.hpp>
#include <iprofile.hpp>
#include <imle.hpp>
#include <ifont.hpp>
#include <ifontdlg.hpp>
#include <stdlib.h>
#define INCL_WINHELP
#include <os2/os2.h>

#include "mainwnd.hpp"
#include "colordlg.hpp"
#include "mainwnd.h"

/**********************************************************/
/*   Global data definitions                             **/
/**********************************************************/

char  * pCh1, * pCh2;
char    binArray[]={0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07,
                    0x08, 0x09, 0x0a, 0x0b, 0x0c, 0x0d, 0x0e, 0x0f};
void   *pMem=malloc(64);

/**********************************************************/
/*   Main application entry point                        **/
/**********************************************************/

void main(int nArg, char* cArg[])
{
 pCh1 = cArg[0];                    /* set the pointers to */
 if (nArg > 1)                      /* args if present     */
   pCh2 = cArg[1];

 AProfWindow mainWindow (ID_MAINWND);
 IApplication::current() .run();
}

/*********************************************************/
/*  Constructor for main window                         **/
/*-------------------------------------------------------*/
/* (1) construct main window and associated controls     */
/* (2) establish command and select message handlers     */
/* (3) control and class data initialization             */
/* (4) select and show the window and client controls    */
/*********************************************************/

AProfWindow :: AProfWindow(unsigned long windowId)
   : IFrameWindow (
     IFrameWindow::defaultStyle()
     | IFrameWindow::minimizedIcon
     | IFrameWindow::accelerator,
     windowId)
  ,menuBar(windowId, this)
  ,isCnv1(ID_CANV1, this, this)
  ,isCnv2(ID_CANV2, &isCnv1, &isCnv1)
  ,listBox(IDW_LISTBOX, &isCnv2, &isCnv2, IRectangle(),
           IListBox::defaultStyle() |
           IControl::tabStop |
           IListBox::noAdjustPosition)
  ,isCnv3(ID_CANV3, &isCnv2, &isCnv2)
  ,listBox2(IDW_LISTBOX2, &isCnv3, &isCnv3, IRectangle(),
           IListBox::defaultStyle() |
           IControl::tabStop |
           IListBox::noAdjustPosition)
  ,mle1(IDW_MLE, &isCnv3, &isCnv3, IRectangle(),
           IMultiLineEdit::defaultStyle() )
  ,isSet1(ID_STCV1, &isCnv1, &isCnv1)
  ,pbDone(IDW_DONE, &isSet1, &isSet1, IRectangle())
  ,pbKey(IDW_KEY,  &isSet1, &isSet1, IRectangle())
  ,pbValue(IDW_VALUE, &isSet1, &isSet1, IRectangle())
{
  ICommandHandler::handleEventsFor(this);
  ICommandHandler::handleEventsFor(&listBox2);
  ICommandHandler::handleEventsFor(&listBox);
  ICommandHandler::handleEventsFor(&mle1);
  ICommandHandler::handleEventsFor(&isSet1);
  ISelectHandler::handleEventsFor(&isCnv2);
  ISelectHandler::handleEventsFor(&isCnv3);

  popUpHandler.handleEventsFor(&listBox);
  popUpHandler.handleEventsFor(&listBox2);
  popUpHandler.handleEventsFor(&mle1);
  popUpHandler.handleEventsFor(&isSet1);

  isCnv1.setOrientation( ISplitCanvas::horizontalSplit );
  isCnv2.setOrientation( ISplitCanvas::verticalSplit );
  isCnv3.setOrientation( ISplitCanvas::horizontalSplit );
  setClient(&isCnv1);

  isCnv1.setSplitWindowPercentage(&isCnv2, 75);
  isCnv1.setSplitWindowPercentage(&isSet1, 15);

  isSet1.setPad(ISize(30, 05));

  pbDone.setText("Done");
  pbDone.setBackgroundColor(IColor(0xff,0x00,0x00));

  pbKey.setText("Keys");
  pbKey.setBackgroundColor(IColor(0x00,0xff,0x00));

  cvMode = 'c';                  /* default is char mode */
  pbValue.setText("Hex");
  pbValue.setBackgroundColor(IColor(0x00,0x00,0xff));

  strcpy(prfName, "*.ini");

  bPrFlag = FALSE;
  ASetPrMenu(bPrFlag);

  memset(pMem, 0, 64);
  memcpy(pMem, binArray, 15);

  setFocus();
  show();
}

/*********************************************************/
/*  Command Handler for main window                     **/
/*-------------------------------------------------------*/
/* Handles WM_COMMAND messages for main window and its   */
/*   controls                                            */
/*********************************************************/

Boolean AProfWindow :: command(ICommandEvent & cmdEvent)
{
 unsigned short value;
 unsigned long  ulMid;

 switch (cmdEvent.commandId())
 {
//
//  Menu Item commands
//
   case IDC1_PROF:    /* select a new profile, using file dialog */
   {
    int x, y=strlen(prfName);

    for (x=y-1; x>0; x--)
      if (prfName[x] == '\\')
      {
        prfName[x+1]=0;            /* terminate string */
        strcat(prfName, "*.ini");  /* concatenate wildcard */
        x=0;                       /* end loop             */
      }

    IFileDialog::Settings fileSettings;
    fileSettings.setTitle("INI file");
    fileSettings.setFileName(prfName);
    IFileDialog fileDialog(this, this, fileSettings);
    if ( fileDialog.pressedOK() )     /* process if OKed */
    {
      if (bPrFlag)
      {
        bPrFlag=FALSE;             /* clean out prev profile */
        delete myCursor;           /* instances,             */
        delete myProfile;          /* if there was one       */
      }
      strcpy(prfName, fileDialog.fileName() );
      myProfile = new IProfile(prfName);               /* create profile */
      myCursor  = new IProfile::Cursor( *myProfile );  /* and cursor     */
      bPrFlag = TRUE;
      ASetPrMenu(bPrFlag);
      ARefrProf();                /* show the apps */
    }
    return(true);
    break;
   }

   case IDP_ADD:      /* "Canned" code for adding some profile */
   {                  /* elements, using both string and       */
                      /* binary data                           */
     void *pBin=malloc(64);
     IString is1("value1");
     IString is2("value2");
     IString is3("value3");
     IString is4(pMem, 32, 0x00);

   /* sample code for populating the profile file */

     myProfile->addOrReplaceElementWithKey(
                  "key1",
                  is1,
                  "testapp");
     myProfile->addOrReplaceElementWithKey(
                  "key2",
                  is2,
                  "testap2");
     myProfile->addOrReplaceElementWithKey(
                  "key3",
                  is3,
                  "testap2");

     myProfile->addOrReplaceElementWithKey(
                  "binkey1",
                  is4,
                  "testbin");

     memset(pBin, 0, 64);
     memcpy(pBin, "1234567", 7);
     IString is5(pBin, 16);
     myProfile->addOrReplaceElementWithKey(
                  "binkey2",
                  is5,
                  "testbin");

   /*--- end of sample code --------------------- */

     return(true);
     break;
   }

   case IDP_REFRESH:    /* refresh APPS listbox from current profile */
   {
     ARefrProf();
     return(TRUE);
     break;
   }

   case IDP_DELETE:    /* delete selected app, keyword combo */
   {
     ULONG   ulSelApp=listBox.selection();
     ULONG   ulSelKey=listBox2.selection();

     myProfile->deleteElementWithKey(listBox2.itemText(ulSelKey).asString(),
                                     listBox.itemText(ulSelApp).asString() );

     ARefrKeys();      /* after delete, refresh keyword list */
     return(TRUE);
     break;
   }

   case IDW_KEY:      /* refresh keywords for selected app */
   {
     ARefrKeys();
     return(TRUE);
     break;
   }

   case IDW_VALUE:   /* refresh value for selected app, keyword */
   {
     if (cvMode == 'c')
     {
       cvMode = 'h';
       pbValue.setText("Char");
     }
     else
     {
       cvMode = 'c';
       pbValue.setText("Hex");
     }

     ARefrValue();
     return(TRUE);
     break;
   }

//   Popup messages
   case IDP_FONT:
   {
    popupOwner = cmdEvent.window();
    IFont cFont(popupOwner);
    IFontDialog::Settings fontSettings(&cFont);
    fontSettings.setTitle("Change Font");
    IFontDialog fontDialog( desktopWindow(),
                            this,
                            IFontDialog::resetButton,
                            fontSettings);
    if (fontDialog.pressedOK() )
    {
     popupOwner->setFont(cFont);
    }
    return(true);
    break;
   }

   case IDP_COLORB:
   {
    ULONG ulRc;

    popupOwner = cmdEvent.window();
    clrDialog = new AClrDialog( "Background", popupOwner );
    ulRc = clrDialog->showModally();
    delete clrDialog;
    return(true);
    break;
   }

   case IDP_COLORF:
   {
    ULONG ulRc;

    popupOwner = cmdEvent.window();
    clrDialog = new AClrDialog( "Foreground", popupOwner );
    ulRc = clrDialog->showModally();
    delete clrDialog;
    return(true);
    break;
   }

//   Exit messages
   case IDC1_EXIT:
   case ID_EXITPROGRAM:
   case IDW_DONE:
    {
     close();
     return(true);
     break;
    }

 }

 return(false);      /* all others, indicate not processed */
}
/*********************************************************/
/*  Select  Handler for main window                     **/
/*-------------------------------------------------------*/
/* Handles WM_CONTROL messages for list boxes            */
/*********************************************************/

Boolean AProfWindow :: selected(IControlEvent & ctrlEvent)
{

 switch (ctrlEvent.controlId())   /* switch on list box ID */
 {
   case IDW_LISTBOX:    /* If APP selected, show its keys  */
   {
      ARefrKeys();
      return(true);
    }

   case IDW_LISTBOX2:   /* If KEY selected, show its value */
   {
      ARefrValue();
      return(true);
    }

   default:
     return(false);
 }

 return(false);  /* all others, indicate not processed */
}
/*********************************************************/
/*  Enable/Disable menu items                           **/
/*********************************************************/
void AProfWindow :: ASetPrMenu(BOOL bFlag)
{
  menuBar.enableItem(IDP_ADD,     bFlag);
  menuBar.enableItem(IDP_DELETE,  bFlag);
  menuBar.enableItem(IDP_SHOW,    bFlag);
  menuBar.enableItem(IDP_REFRESH, bFlag);
}
/*********************************************************/
/*  Refresh Apps                                        **/
/* ----------------------------------------------------- */
/* This routine:                                         */
/*   (1) clears value control and keyword listbox        */
/*   (2) clears app listbox                              */
/*   (3) uses app cursor to access each app name and     */
/*       display it in the app listbox                   */
/*                                                       */
/*********************************************************/
void AProfWindow :: ARefrProf(void)
{
  mle1.removeAll();                 /* clear value     */
  listBox2.removeAll();             /*  and keys       */
  listBox.removeAll();
  for ( myCursor->setToFirst();
        myCursor->isValid();
        myCursor->setToNext()  )
    listBox.addAscending(
          (myProfile->applicationOrKeyAt( *myCursor ) ).asString()
                        );
}
/*********************************************************/
/*  Refresh Keys                                        **/
/* ----------------------------------------------------- */
/* This routine:                                         */
/*   (1) clears keyword listbox                          */
/*   (2) creates new instance of keyword cursor          */
/*   (3) uses the cursor to access each key name for     */
/*       selected app and displays it in key listbox     */
/*   (4) clears value multi-line edit control            */
/*                                                       */
/*********************************************************/
void AProfWindow :: ARefrKeys(void)
{
  ULONG   ulSelect=listBox.selection();  /* index of selected app */

  IProfile::Cursor appCursor( *myProfile,
                              listBox.itemText(ulSelect).asString() );

  listBox2.removeAll();
  for ( appCursor.setToFirst();
        appCursor.isValid();
        appCursor.setToNext()  )
    listBox2.addAscending(
          (myProfile->applicationOrKeyAt( appCursor ) ).asString()
                         );
   mle1.removeAll();

}
/*********************************************************/
/*  Refresh Value                                       **/
/* ----------------------------------------------------- */
/* This routine:                                         */
/*   (1) clears multi-line edit value control            */
/*   (2) displays, as string, the value of the currently */
/*       selected app and keyword                        */
/*                                                       */
/*********************************************************/
void AProfWindow :: ARefrValue(void)
{
 ULONG   ulSelApp=listBox.selection();  /* index of selected app */
 ULONG   ulSelKey=listBox2.selection(); /* index of selected keyword */

 mle1.removeAll();

 if (cvMode == 'c')
   mle1.setText( myProfile->elementWithKey( listBox2.itemText(ulSelKey).asString(),
                                            listBox.itemText(ulSelApp).asString()
                                          ).asString() );
 else
   mle1.setText( myProfile->elementWithKey( listBox2.itemText(ulSelKey).asString(),
                                            listBox.itemText(ulSelApp).asString()
                                          ).c2x() );

}

IBase :: Boolean APopUpHandler::makePopUpMenu(IMenuEvent &menuEvent)
{
  IPopUpMenu *popUpMenu;
  popUpOwner = menuEvent.window();

  popUpMenu = new IPopUpMenu(IDP_POPUP, popUpOwner);
  if (popUpMenu)
  {
    popUpMenu->setAutoDeleteObject();
    popUpMenu->show(menuEvent.mousePosition() );
  }
  return(true);
} /* end APopUpHandler */
